@@ -126,7 +126,6 @@ class ImpulseGraph(object):
         self.graph = {}  # dictionary for graph attributes
         self._node = {}
         self._adj = {}
-        self.edgeid = 0
 
         self.graph.update(attr)
 
@@ -206,7 +205,7 @@ class ImpulseGraph(object):
         """
         if len(self.tree.keys()) == 0:
             raise IndexError("ImpulseGraph is empty.")
-        return self.tree.keys()[0], self.tree.keys()[-1]
+        return list(self.tree.keys())[0], list(self.tree.keys())[-1]
 
     def add_node(self, node_for_adding, **attr):
         """Add a single node `node_for_adding` and update node attributes.
@@ -292,7 +291,7 @@ class ImpulseGraph(object):
             else:
                 self.add_node(n, **attr)
 
-    def number_of_nodes(self, begin=None, end=None, inclusive=(True, True)):
+    def number_of_nodes(self, begin=None, end=None, inclusive=(True, False)):
         """Return the number of nodes in the impulse graph between the given interval.
 
         Parameters
@@ -324,11 +323,13 @@ class ImpulseGraph(object):
         >>> G.number_of_nodes(begin=5, end=8)
         2
         >>> G.number_of_nodes(end=11)
-        4
+        2
         """
 
         if begin is None and end is None:
             return len(self._node)
+        if end is None:
+            inclusive = (inclusive[0], True)
 
         inodes = set()
         for edge in self.__search_tree(begin, end, inclusive=inclusive):
@@ -337,7 +338,7 @@ class ImpulseGraph(object):
 
         return len(inodes)
 
-    def has_node(self, n, begin=None, end=None, inclusive=(True, True)):
+    def has_node(self, n, begin=None, end=None, inclusive=(True, False)):
         """Return True if the impulse graph contains the node n, during the given interval.
 
         Identical to `n in G` when 'begin' and 'end' are not defined.
@@ -386,7 +387,7 @@ class ImpulseGraph(object):
                 return True
         return False
 
-    def nodes(self, begin=None, end=None, inclusive=(True, True), data=False, default=None):
+    def nodes(self, begin=None, end=None, inclusive=(True, False), data=False, default=None):
         """A NodeDataView of the ImpulseGraph nodes.
 
         A nodes is considered to be present during an interval, if it has
@@ -455,7 +456,7 @@ class ImpulseGraph(object):
 
         return NodeDataView(node_dict, data=data, default=default)
 
-    def remove_node(self, n, begin=None, end=None, inclusive=(True, True)):
+    def remove_node(self, n, begin=None, end=None, inclusive=(True, False)):
         """Remove the presence of a node n within the given interval.
 
         Removes the presence node n and all adjacent edges within the given interval.
@@ -493,20 +494,23 @@ class ImpulseGraph(object):
             return
 
         if begin is None and end is None:
-            for iedge in list(self._adj[n].keys()):
-                self.__remove_iedge(iedge)
+            self._adj.pop(n, None)
+            for v in self._adj:
+                if n in self._adj[v]:
+                    self._adj[v].pop(n, None)
+
         else:
             remove_edges = []
-            for edge in self.__search_tree(begin, end, inclusive=inclusive):
+            for edge in self.__search_tree(begin, end, inclusive):
                 if edge[0] == n or edge[1] == n:
                     remove_edges.append(edge)
             for edge in remove_edges:
                 self.__remove_iedge(edge)
 
         # delete the node and its attributes if no edge left
-        if len(self._adj[n]) == 0:
-            del self._adj[n]
-            del self._node[n]
+        if n not in self._adj or len(self._adj[n]) == 0:
+            self._adj.pop(n, None)
+            self._node.pop(n, None)
 
     def add_edge(self, u, v, t, **attr):
         """Add an edge between u and v, at t.
@@ -559,15 +563,12 @@ class ImpulseGraph(object):
         >>> G.add_edge(1, 3, 9, weight=7, capacity=15, length=342.7)
         """
 
-        eid = self.edgeid
-        self.tree.setdefault(t, set()).add((u, v, eid))
+        self.tree.setdefault(t, set()).add((u, v))
 
         self._node.setdefault(u, {})
         self._node.setdefault(v, {})
-        self._adj.setdefault(u, {})[(u, v, eid, t)] = attr
-        self._adj.setdefault(v, {})[(u, v, eid, t)] = attr
-
-        self.edgeid += 1
+        self._adj.setdefault(u, {}).setdefault(v, {})[(u, v, t)] = attr
+        self._adj.setdefault(v, {}).setdefault(u, {})[(u, v, t)] = attr
 
     def add_edges_from(self, ebunch_to_add, **attr):
         """Add all the edges in ebunch_to_add.
@@ -607,7 +608,7 @@ class ImpulseGraph(object):
                 raise NetworkXError("Edge tuple {0} must be a 3-tuple.".format(e))
             self.add_edge(e[0], e[1], e[2], **attr)
 
-    def has_edge(self, u, v, begin=None, end=None, inclusive=(True, True)):
+    def has_edge(self, u, v, begin=None, end=None, inclusive=(True, False)):
         """Return True if there exists an edge between u and v
         in the impulse graph, during the given interval.
 
@@ -632,21 +633,22 @@ class ImpulseGraph(object):
         >>> G.has_edge(2, 4, begin=12)
         False
         """
-
-        if begin is None and end is None:
-            for edge in self._adj[u].keys():
-                if edge[1] == v:
-                    return True
+        if u not in self._adj or v not in self._adj[u]:
             return False
 
-        begin, end = self.__validate_interval(begin, end)
+        if begin is None and end is None:
+            return True
 
-        for edge in self._adj[u].keys():
-            if edge[1] == v and self.__in_interval(edge[3], begin, end, inclusive=inclusive):
+        if begin and end and begin > end:
+            raise NetworkXError("IntervalGraph: interval end must be bigger than or equal to begin: "
+                                "begin: {}, end: {}.".format(begin, end))
+
+        for iv in self._adj[u][v]:
+            if self.__in_interval(iv[2], begin, end, inclusive=inclusive):
                 return True
         return False
 
-    def edges(self, u=None, v=None, begin=None, end=None, inclusive=(True, True), data=False, default=None):
+    def edges(self, u=None, v=None, begin=None, end=None, inclusive=(True, False), data=False, default=None):
         """Returns a list of Interval objects of the ImpulseGraph edges.
 
         All edges which are present within the given interval.
@@ -729,27 +731,53 @@ class ImpulseGraph(object):
         [((1, 3, 4), 8)]
         """
 
-        iedges = []
-        for edge in self.__search_tree(begin, end, inclusive=inclusive):
-            if u is not None and v is not None and u == edge[0] and v == edge[1]:
-                iedges.append(edge)
-            elif (u is not None and v is None) and (u == edge[0] or u == edge[1]):
-                iedges.append(edge)
-            elif (u is None and v is not None) and (v == edge[0] or v == edge[1]):
-                iedges.append(edge)
-            elif u is None and v is None:
-                iedges.append(edge)
+        if begin is None:
+            inclusive = (True, inclusive[1])
+        if end is None:
+            inclusive = (inclusive[0], True)
+
+        if not u and not v:
+            if not begin or not end:
+                iedges = [iv for iv in self.__search_tree(begin, end, inclusive)]
+            # interval filtering
+            else:
+                if begin and end and begin > end:
+                    raise NetworkXError("IntervalGraph: interval end must be bigger than or equal to begin: "
+                                        "begin: {}, end: {}.".format(begin, end))
+                iedges = [iv for iv in self.__search_tree(begin, end, inclusive)]
+
+        else:
+            # Node filtering
+            if u and v:
+                if u not in self._adj:
+                    return []
+                if v not in self._adj[u]:
+                    return []
+                iedges = self._adj[u][v]
+
+            elif u is not None:
+                if u not in self._adj:
+                    return []
+                iedges = [iv for v in self._adj[u] for iv in self._adj[u][v]]
+            else:
+                if v not in self._adj:
+                    return []
+                iedges = [iv for u in self._adj[v] for iv in self._adj[v][u]]
+
+            # Interval filtering
+            if begin and end and begin > end:
+                raise NetworkXError("IntervalGraph: interval end must be bigger than or equal to begin: "
+                                    "begin: {}, end: {}.".format(begin, end))
+            iedges = [iv for iv in iedges if self.__in_interval(iv[2], begin, end, inclusive=inclusive)]
 
         if data is False:
-            return [(edge[0], edge[1], edge[3]) for edge in iedges]
+            return [edge for edge in iedges]
 
         if data is True:
-            return [((edge[0], edge[1], edge[3]), self._adj[edge[0]][edge]) for edge in iedges]
+            return [(edge, self._adj[edge[0]][edge[1]][edge]) for edge in iedges]
+        return [(edge, self._adj[edge[0]][edge[1]][edge][data]) if data in self._adj[edge[0]][edge[1]][edge] else (edge, default) for edge in iedges]
 
-        return [((edge[0], edge[1], edge[3]), self._adj[edge[0]][edge][data]) if data in self._adj[edge[0]][edge]
-                else ((edge[0], edge[1], edge[3]), default) for edge in iedges]
-
-    def remove_edge(self, u, v, begin=None, end=None, inclusive=(True, True)):
+    def remove_edge(self, u, v, begin=None, end=None, inclusive=(True, False)):
         """Remove the edge between u and v in the impulse graph,
         during the given interval.
 
@@ -782,15 +810,36 @@ class ImpulseGraph(object):
         True
         """
 
-        iedges = []
-        for edge in self.__search_tree(begin, end, inclusive=inclusive):
-            if u == edge[0] and v == edge[1]:
-                iedges.append(edge)
+        if u not in self._adj or v not in self._adj[u]:
+            return
 
-        for edge in iedges:
+        iedges_to_remove = []
+
+        # remove every edge between u and v
+        if begin is None and end is None:
+            for iv in self._adj[u][v]:
+                iedges_to_remove.append(iv)
+
+        else:
+            for iv in self._adj[u][v]:
+                if self.__in_interval(iv[2], begin, end):
+                    iedges_to_remove.append(iv)
+
+        # removing found iedges
+        for edge in iedges_to_remove:
             self.__remove_iedge(edge)
 
-    def degree(self, node=None, begin=None, end=None, delta=False, inclusive=(True, True)):
+        # clean up empty dictionaries
+        if len(self._adj[u][v]) == 0:
+            self._adj[u].pop(v, None)
+        if len(self._adj[v][u]) == 0:
+            self._adj[v].pop(u, None)
+        if len(self._adj[u]) == 0:
+            self._adj.pop(u, None)
+        if len(self._adj[v]) == 0:
+            self._adj.pop(v, None)
+
+    def degree(self, node=None, begin=None, end=None, delta=False, inclusive=(True, False)):
         """Return the degree of a specified node between time begin and end.
 
         Parameters
@@ -816,15 +865,17 @@ class ImpulseGraph(object):
         >>> G.add_edge(2, 3, 8)
         >>> G.degree(2)
         2
-        >>> G.degree(2,2)
+        >>> G.degree(2, 2)
         2
-        >>> G.degree(2,end=8)
+        >>> G.degree(2, end=8)
         1
         >>> G.degree()
         1.33333
-        >>> G.degree(2,delta=True)
+        >>> G.degree(2, delta=True)
         [(3, 1), (8, 1)]
         """
+        if end is None:
+            inclusive = (inclusive[0], True)
         # no specified node, return mean degree
         if node == None:
             n = 0
@@ -848,7 +899,7 @@ class ImpulseGraph(object):
         output = []
 
         # for each edge determine if the begin and/or end value is in specified time period
-        for edge in self.edges(u=node, begin=begin, end=end, inclusive=(True, True)):
+        for edge in self.edges(u=node, begin=begin, end=end, inclusive=inclusive):
             d.setdefault(edge[2], []).append((edge[0], edge[1]))
 
         # for each time in Dict add to output list the len of each value
@@ -867,18 +918,13 @@ class ImpulseGraph(object):
         iedge : Edge Tuple (u,v,eid,t)
             Edge to be removed.
         """
-
-        u = iedge[0]
-        v = iedge[1]
-        eid = iedge[2]
-        t = iedge[3]
-
         try:
-            self.tree[t].remove((u, v, eid))
-            del self._adj[u][iedge]
-            del self._adj[v][iedge]
+            self.tree[iedge[2]].remove((iedge[0], iedge[1]))
+            del self._adj[iedge[0]][iedge[1]][iedge]
+            del self._adj[iedge[1]][iedge[0]][iedge]
         except:
             return
+        print(self._adj)
 
     def __validate_interval(self, begin=None, end=None):
         """Returns validated begin and end.
@@ -896,7 +942,7 @@ class ImpulseGraph(object):
 
         return begin, end
 
-    def __search_tree(self, begin=None, end=None, inclusive=(True, True)):
+    def __search_tree(self, begin=None, end=None, inclusive=(True, False)):
         """if begin and end are equal performs a point search on the tree,
         otherwise an interval search is performed.
 
@@ -917,7 +963,7 @@ class ImpulseGraph(object):
             for edge in self.tree[t]:
                 yield (*edge, t)
 
-    def __in_interval(self, t, begin, end, inclusive=(True, True)):
+    def __in_interval(self, t, begin, end, inclusive=(True, False)):
         """
         Parameters
         ----------
@@ -947,7 +993,17 @@ class ImpulseGraph(object):
         if inclusive == (False, False):
             return t > begin and t < end
 
-    def to_subgraph(self, begin, end, inclusive=(True, True), multigraph=False, edge_data=False,
+    def to_networkx_graph(self, begin, end, inclusive=(True, False), multigraph=False, edge_data=False,
+                          edge_timestamp_data=False, node_data=False):
+        """Return a networkx Graph or MultiGraph which includes all the nodes and
+        edges which have timestamps within the given interval.
+
+        Wrapper function for ImpulseGraph.to_subgraph. Refer to ImpulseGraph.to_subgraph for full description.
+        """
+        return self.to_subgraph(begin=begin, end=end, inclusive=inclusive, multigraph=multigraph, edge_data=edge_data,
+                                edge_timestamp_data=edge_timestamp_data, node_data=node_data)
+
+    def to_subgraph(self, begin, end, inclusive=(True, False), multigraph=False, edge_data=False,
                     edge_timestamp_data=False, node_data=False):
         """Return a networkx Graph or MultiGraph which includes all the nodes and
         edges which have timestamps within the given interval.
@@ -1013,10 +1069,10 @@ class ImpulseGraph(object):
             G.add_edges_from((iedge[0], iedge[1], dict(self._adj[iedge[0]][iedge], timestamp=iedge[3]))
                              for iedge in iedges)
         elif edge_data:
-            G.add_edges_from((iedge[0], iedge[1], self._adj[iedge[0]][iedge])
+            G.add_edges_from((iedge[0], iedge[1], self._adj[iedge[0]][iedge[1]][iedge])
                              for iedge in iedges)
         elif edge_timestamp_data:
-            G.add_edges_from((iedge[0], iedge[1], {'timestamp': iedge[3]})
+            G.add_edges_from((iedge[0], iedge[1], {'timestamp': iedge[2]})
                              for iedge in iedges)
         else:
 
@@ -1027,7 +1083,7 @@ class ImpulseGraph(object):
 
         return G
 
-    def to_snapshots(self, number_of_snapshots, multigraph=False, edge_data=False, edge_timestamp_data=False,
+    def to_snapshots(self, number_of_snapshots=False, length_of_snapshots=False, multigraph=False, edge_data=False, edge_timestamp_data=False,
                      node_data=False, return_length=False):
         """Return a list of networkx Graph or MultiGraph objects as snapshots
         of the impulse graph in consecutive order.
@@ -1036,6 +1092,9 @@ class ImpulseGraph(object):
         ----------
         number_of_snapshots : integer
             Number of snapshots to divide the interval graph into.
+            Must be bigger than 2.
+        length_of_snapshots : integer or float
+            Length of snapshots to divide the interval graph into.
             Must be bigger than 1.
         multigraph : bool, optional (default= False)
             If True, a networkx MultiGraph will be returned. If False, networkx Graph.
@@ -1053,6 +1112,7 @@ class ImpulseGraph(object):
         See Also
         --------
         to_subgraph : subgraph based on an interval
+        to_snapshotgraph : snapshotgraph based on impulse graph
 
         Notes
         -----
@@ -1091,29 +1151,105 @@ class ImpulseGraph(object):
         [(6, 4, {'timestamp': 19})]
         """
 
-        if number_of_snapshots < 2 or type(number_of_snapshots) is not int:
-            raise NetworkXError("ImpulseGraph: number of snapshots must be an integer and 2 or bigger. "
-                                "{0} was passed.".format(number_of_snapshots))
+        if type(number_of_snapshots) is bool and type(length_of_snapshots) is bool:
+            raise NetworkXError("IntervalGraph: either number of snapshots or length of snapshots must be given.")
+
+        if type(number_of_snapshots) is not bool and type(length_of_snapshots) is not bool:
+            raise NetworkXError("IntervalGraph: either number of snapshots or length of snapshots must be given, "
+                                "not both.")
 
         begin, end = self.interval()
-        snapshot_len = (end - begin) / number_of_snapshots
+        if type(length_of_snapshots) is int or type(length_of_snapshots) is float:
+            if length_of_snapshots < 0:
+                raise NetworkXError("IntervalGraph: length of snapshots must be bigger than 0. "
+                                    "{0} was passed.".format(number_of_snapshots))
+            number_of_snapshots = (end - begin) // length_of_snapshots
+            if (end - begin) % length_of_snapshots != 0:
+                number_of_snapshots += 1
+
+        if number_of_snapshots < 2 or type(number_of_snapshots) is not int:
+            raise NetworkXError("IntervalGraph: number of snapshots must be an integer and 2 or bigger. "
+                                "{0} was passed.".format(number_of_snapshots))
+
+        if length_of_snapshots is False:
+            length_of_snapshots = (end - begin) / number_of_snapshots
 
         snapshots = []
-        end_inclusive_addition = 0
+        inclusive = (True, False)
         for i in range(number_of_snapshots):
-            # since to_subgraph is end non-inclusive, shift the end up by 1 to include end in the last snapshot.
             if i == number_of_snapshots - 1:
-                end_inclusive_addition = 1
-
+                inclusive = (True, True)
             snapshots.append(
-                self.to_subgraph(begin + snapshot_len * i, begin + snapshot_len * (i + 1) + end_inclusive_addition,
+                self.to_subgraph(begin + length_of_snapshots * i,
+                                 begin + length_of_snapshots * (i + 1), inclusive=inclusive,
                                  multigraph=multigraph, edge_data=edge_data, edge_timestamp_data=edge_timestamp_data,
                                  node_data=node_data))
         if return_length:
-            return snapshots, snapshot_len
+            return snapshots, length_of_snapshots
 
         return snapshots
 
+    def to_snapshot_graph(self, multigraph=False, edge_data=False, edge_timestamp_data=False, node_data=False):
+        """
+        Return a dnx.SnapshotGraph of the impulse graph.
+
+        Parameters
+        ----------
+        multigraph : bool, optional (default= False)
+            If True, a networkx MultiGraph will be returned. If False, networkx Graph.
+        edge_data: bool, optional (default= False)
+            If True, edges will keep their attributes.
+        edge_timestamp_data : bool, optional (default= False)
+            If True, each edge's attribute will also include its timestamp data.
+            If `edge_data= True` and there already exist edge attributes named timestamp
+            it will be overwritten.
+        node_data : bool, optional (default= False)
+            if True, each node's attributes will be included.
+
+        See Also
+        --------
+        to_snapshots : divide the impulse graph to snapshots
+
+        Notes
+        -----
+        In order to create snapshots, timestamp of edges of the impulse graph must be numbers.
+
+        If multigraph= False, and edge_data=True or edge_timestamp_data=True,
+        in case there are multiple edges, only one will show with one of the edge's attributes.
+
+        Examples
+        --------
+        Snapshots of NetworkX Graph
+
+        >>> G = dnx.ImpulseGraph()
+        >>> G.add_edges_from([(1, 2, 10), (2, 3, 11), (2, 4, 11), (4, 6, 19)])
+        >>> S = G.to_snapshot_graph(edge_timestamp_data=True)
+        >>> for g in S:
+        >>> ... g.edges(data=True))
+        [(1, 2, {'timestamp': 10})]
+        [(2, 3, {'timestamp': 11}), (2, 4, {'timestamp': 11})]
+        [(4, 6, {'timestamp': 19})]
+
+        Snapshots of NetworkX MultiGraph
+
+        >>> S = G.to_snapshot_graph(multigraph=True, edge_timestamp_data=True)
+        >>> for g in S:
+        >>> ... g.edges(data=True))
+        [(1, 2, {'timestamp': 10}), (1, 2, {'timestamp': 10})]
+        [(2, 3, {'timestamp': 11}), (2, 3, {'timestamp': 11}), (2, 4, {'timestamp': 11}), (2, 4, {'timestamp': 11})]
+        [(4, 6, {'timestamp': 19}), (4, 6, {'timestamp': 19})]
+        """
+
+        G = dnx.SnapshotGraph()
+
+        for time, snapshot in self.tree.items():
+            G.insert(graph=self.to_subgraph(time, time, inclusive=(True, True), multigraph=multigraph,
+                                            edge_data=edge_data, edge_timestamp_data=edge_timestamp_data,
+                                            node_data=node_data),
+                     time=time)
+
+        return G
+
     @staticmethod
     def from_networkx_graph(graph, timestamp='timestamp'):
         """Convert a NetworkX Graph to a ImpulseGraph.
@@ -1154,11 +1290,11 @@ class ImpulseGraph(object):
         return G
 
     @staticmethod
-    def load_from_txt(path, delimiter=" ", nodetype=int, timestamptype=float, comments="#"):
+    def load_from_txt(path, delimiter=" ", nodetype=int, timestamptype=float, order=('u', 'v', 't'), comments="#"):
         """Read impulse graph in from path.
-           Every line in the file must be an edge in the following format: "node node timestamp".
            Timestamps must be integers or floats.
            Nodes can be any hashable objects.
+           Edge Attributes can be assigned with in the following format: Key=Value
 
         Parameters
         ----------
@@ -1172,6 +1308,9 @@ class ImpulseGraph(object):
         Convert timestamp to this type.
         This must be an orderable type, ideally int or float. Other orderable types have not been fully tested.
 
+        order : Python 3-tuple, optional (default= ('u', 'v', 't'))
+        This must be a 3-tuple containing strings 'u', 'v', and 't'. 'u' specifies the starting node, 'v' the ending node, and 't' the timestamp.
+
         comments : string, optional
            Marker for comment lines
 
@@ -1204,6 +1343,9 @@ class ImpulseGraph(object):
         if delimiter == '=':
             raise ValueError("Delimiter cannot be =.")
 
+        if len(order) != 3 or 'u' not in order or 'v' not in order or 't' not in order:
+            raise ValueError("Order must be a 3-tuple containing strings 'u', 'v', and 't'.")
+
         with open(path, 'r') as file:
             for line in file:
                 p = line.find(comments)
@@ -1214,9 +1356,9 @@ class ImpulseGraph(object):
 
                 line = line.rstrip().split(delimiter)
 
-                u = line[0]
-                v = line[1]
-                t = line[2]
+                u = line[order.index('u')]
+                v = line[order.index('v')]
+                t = line[order.index('t')]
 
                 edgedata = {}
                 for data in line[3:]:
