@@ -2,7 +2,9 @@ from networkx import NetworkXError
 from sortedcontainers import SortedDict
 
 from dynetworkx.classes.intervalgraph import IntervalGraph
-from intervaltree import IntervalTree, Interval
+from dynetworkx.classes.intervaltree import IntervalTree
+from networkx.classes.digraph import DiGraph
+from networkx.classes.multidigraph import MultiDiGraph
 
 
 class IntervalDiGraph(IntervalGraph):
@@ -83,20 +85,20 @@ class IntervalDiGraph(IntervalGraph):
         >>> G.add_edge(1, 3, 4, 9, weight=7, capacity=15, length=342.7)
         """
 
-        iedge = self.__get_iedge_in_tree(u, v, begin, end)
+        iedge = self.edges(u, v, begin, end, data=True)
 
         # if edge exists, just update attr
-        if iedge is not None:
-            self._pred[u][iedge].update(attr)
-            self._succ[v][iedge].update(attr)
+        if u in self._pred and v in self._succ and v in self._pred[u] and u in self._succ[v] and (u, v, begin, end) in self._pred[u][v] and (u, v, begin, end) in self._succ[v][u]:
+            self._pred[u][(u, v, begin, end)].update(attr)
+            self._succ[v][(u, v, begin, end)].update(attr)
             return
 
-        iedge = Interval(begin, end, (u, v))
+        iedge = (u, v, begin, end)
 
         # add nodes
 
-        self._pred.setdefault(u, {})
-        self._succ.setdefault(v, {})
+        self._pred.setdefault(u, {}).setdefault(v, {})
+        self._succ.setdefault(v, {}).setdefault(u, {})
         self._node.setdefault(u, {})
         self._node.setdefault(v, {})
 
@@ -106,7 +108,7 @@ class IntervalDiGraph(IntervalGraph):
         except ValueError:
             raise NetworkXError("IntervalDiGraph: edge duration must be strictly bigger than zero {0}.".format(iedge))
 
-        self._pred[u][iedge] = self._succ[v][iedge] = attr
+        self._pred[u][v][iedge] = self._succ[v][u][iedge] = attr
 
     def add_edges_from(self, ebunch_to_add, **attr):
         """Add all the edges in ebunch_to_add.
@@ -197,22 +199,28 @@ class IntervalDiGraph(IntervalGraph):
         False
         """
 
+        if u not in self._pred:
+            return False
+        if v not in self._pred[u]:
+            return False
+
         if begin is None and end is None:
-            for iv in self._pred[u].keys():
-                if iv.data[1] == v:
-                    return True
+            if v in self._pred[u]:
+                return True
             return False
 
         if not overlapping:
             if begin is None or end is None:
                 raise NetworkXError("For exact interval match (overlapping=False), both begin and end must be defined.")
 
-            return self.__get_iedge_in_tree(u, v, begin, end) is not None
+            return self.edges(u, v, begin, end) is not None
 
-        begin, end = self.__validate_interval(begin, end)
+        if begin and end and begin > end:
+            raise NetworkXError("IntervalGraph: interval end must be bigger than or equal to begin: "
+                                "begin: {}, end: {}.".format(begin, end))
 
-        for iv in self._pred[u].keys():
-            if iv.data[1] == v and iv.overlaps(begin=begin, end=end):
+        for iv in self._pred[u][v]:
+            if self.__overlaps_or_contains(iv, begin, end):
                 return True
         return False
 
@@ -263,25 +271,25 @@ class IntervalDiGraph(IntervalGraph):
         >>> G = dnx.IntervalDiGraph()
         >>> G.add_edges_from([(1, 2, 3, 10), (2, 4, 1, 11), (6, 4, 12, 19), (2, 4, 8, 15)])
         >>> G.edges()
-        [Interval(8, 15, (2, 4)), Interval(3, 10, (1, 2)), Interval(1, 11, (2, 4)), Interval(12, 19, (6, 4))]
+        [(1, 2, 3, 10), (2, 4, 1, 11), (6, 4, 12, 19), (2, 4, 8, 15)]
 
         To get edges which appear in a specific interval:
 
         >>> G.edges(begin=10)
-        [Interval(12, 19, (6, 4)), Interval(1, 11, (2, 4)), Interval(8, 15, (2, 4))]
+        [(1, 2, 3, 10), (2, 4, 1, 11), (6, 4, 12, 19), (2, 4, 8, 15)]
         >>> G.edges(end=5)
-        [Interval(3, 10, (1, 2)), Interval(1, 11, (2, 4))]
+        [(1, 2, 3, 10), (2, 4, 1, 11)]
         >>> G.edges(begin=2, end=4)
-        [Interval(3, 10, (1, 2)), Interval(1, 11, (2, 4))]
+        (1, 2, 3, 10), (2, 4, 1, 11)
 
         To get edges with either of the two nodes being defined:
 
         >>> G.edges(u=2)
-        [Interval(3, 10, (1, 2)), Interval(1, 11, (2, 4)), Interval(8, 15, (2, 4))]
+        [(2, 4, 1, 11), (2, 4, 8, 15)]
         >>> G.edges(u=2, begin=11)
-        [Interval(1, 11, (2, 4)), Interval(8, 15, (2, 4))]
+        [(2, 4, 8, 15)]
         >>> G.edges(u=2, v=4, end=8)
-        [Interval(1, 11, (2, 4))]
+        [(2, 4, 1, 11)]
         >>> G.edges(u=1, v=6)
         []
 
@@ -292,35 +300,49 @@ class IntervalDiGraph(IntervalGraph):
         >>> G.add_edge(1, 2, 3, 10, weight=10)
         >>> G.add_edge(2, 6, 2, 10)
         >>> G.edges(data="weight")
-        [(Interval(2, 8, (2, 3)), None), (Interval(3, 10, (1, 2)), 10), (Interval(1, 4, (1, 3)), 8)]
+        [(2, 6, 2, 10)), None), (1, 2, 3, 10), 10), (1, 3, 1, 4), 8)]
         >>> G.edges(data="weight", default=5)
-        [(Interval(2, 8, (2, 3)), 5), (Interval(3, 10, (1, 2)), 10), (Interval(1, 4, (1, 3)), 8)]
+        [(2, 6, 2, 10)), 5), (1, 2, 3, 10), 10), (1, 3, 1, 4), 8)]
         >>> G.edges(data=True)
-        [(Interval(2, 8, (2, 3)), {}), (Interval(3, 10, (1, 2)), {'weight': 10}), (Interval(1, 4, (1, 3)), {'height': 18, 'weight': 8})]
+        [(2, 6, 2, 10)), {}), (1, 2, 3, 10), {weight:10}), (1, 3, 1, 4), {weight:8, height:18})]
         >>> G.edges(u=1, begin=5, end=9, data="weight")
-        [(Interval(3, 10, (1, 2)), 10)]
+        [(1, 2, 3, 10), 10)]
         """
         # If non of the nodes are defined the interval tree is queried for the list of edges,
         # otherwise the edges are returned based on the nodes in the self._adj.o
-        iedges = []
-        if u is None and v is None:
-            if begin is None and end is None:
-                iedges = self.tree.all_intervals
+
+        if not u and not v:
+            if not begin and not end:
+                iedges = self.tree[None:None]
             # interval filtering
             else:
-                iedges = self.__search_tree(begin, end)
+                if begin and end and begin > end:
+                    raise NetworkXError("IntervalGraph: interval end must be bigger than or equal to begin: "
+                                        "begin: {}, end: {}.".format(begin, end))
+                iedges = self.tree[begin:end]
 
         else:
             # Node filtering
-            if u is not None and v is not None:
-                iedges = [iv for iv in self._pred[u].keys() if iv.data[1] == v]
-            elif u is not None and u in self._pred:
-                iedges = self._pred[u].keys()
-            elif v in self._succ:
-                iedges = self._succ[v].keys()
+            if u and v:
+                if u not in self._pred:
+                    return []
+                if v not in self._succ:
+                    return []
+
+                iedges = self._pred[u][v]
+            elif u:
+                if u not in self._pred:
+                    return []
+                iedges = [iv for v in self._pred[u] for iv in self._pred[u][v]]
+            else:
+                if v not in self._succ:
+                    return []
+                iedges = [iv for u in self._succ[v] for iv in self._succ[v][u]]
 
             # Interval filtering
-            begin, end = self.__validate_interval(begin, end)
+            if begin and end and begin > end:
+                raise NetworkXError("IntervalGraph: interval end must be bigger than or equal to begin: "
+                                    "begin: {}, end: {}.".format(begin, end))
             iedges = [iv for iv in iedges if IntervalDiGraph.__overlaps_or_contains(iv, begin, end)]
 
         # Appending attribute data if needed
@@ -328,9 +350,9 @@ class IntervalDiGraph(IntervalGraph):
             return iedges if isinstance(iedges, list) else list(iedges)
 
         if data is True:
-            return [(iv, self._pred[iv.data[0]][iv]) for iv in iedges]
+            return [(iv, self._pred[iv[0]][iv[1]][iv]) for iv in iedges]
 
-        return [(iv, self._pred[iv.data[0]][iv][data]) if data in self._pred[iv.data[0]][iv].keys() else
+        return [(iv, self._pred[iv[0]][iv[1]][iv][data]) if data in self._pred[iv[0]][iv[1]][iv] else
                 (iv, default) for iv in iedges]
 
     def remove_edge(self, u, v, begin=None, end=None, overlapping=True):
@@ -386,12 +408,16 @@ class IntervalDiGraph(IntervalGraph):
         >>> G.has_edge(2, 4, begin=1, end=11)
         False
         """
+
+        if u not in self._pred or v not in self._pred[u]:
+            return
+
         # remove edge between u and v with the exact given interval
         if not overlapping:
             if begin is None or end is None:
                 raise NetworkXError("For exact interval match (overlapping=False), both begin and end must be defined.")
 
-            iedge = self.__get_iedge_in_tree(u, v, begin, end)
+            iedge = self.edges(u, v, begin, end)
             if iedge is not None:
                 self.__remove_iedge(iedge)
             return
@@ -400,21 +426,33 @@ class IntervalDiGraph(IntervalGraph):
 
         # remove every edge between u and v
         if begin is None and end is None:
-            for iv in self._pred[u].keys():
-                if iv.data[1] == v:
-                    iedges_to_remove.append(iv)
+            for iv in self._pred[u][v]:
+                iedges_to_remove.append(iv)
 
         # remove edge between u and v with overlapping interval with the given interval
-        begin, end = self.__validate_interval(begin, end)
+        else:
+            if begin and end and begin > end:
+                raise NetworkXError("IntervalGraph: interval end must be bigger than or equal to begin: "
+                                "begin: {}, end: {}.".format(begin, end))
 
-        for iv in self._pred[u].keys():
-            if iv.data[1] == v and IntervalDiGraph.__overlaps_or_contains(iv, begin, end):
-                iedges_to_remove.append(iv)
+            for iv in self._pred[u][v]:
+                if IntervalDiGraph.__overlaps_or_contains(iv, begin, end):
+                    iedges_to_remove.append(iv)
 
         # removing found iedges
         for iv in iedges_to_remove:
             self.__remove_iedge(iv)
 
+        # clean up empty dictionaries
+        if len(self._pred[u][v]) == 0:
+            self._pred[u].pop(v, None)
+        if len(self._succ[v][u]) == 0:
+            self._succ[v].pop(u, None)
+        if len(self._pred[u]) == 0:
+            self._pred.pop(u, None)
+        if len(self._succ[v]) == 0:
+            self._succ.pop(v, None)
+
     def degree(self, node=None, begin=None, end=None, delta=False):
         """Return the degree of a specified node between time begin and end.
 
@@ -468,9 +506,9 @@ class IntervalDiGraph(IntervalGraph):
 
         # delta == True, return list of changes
         if begin == None:
-            begin = self.tree.begin()
+            begin = self.tree.begin
         if end == None:
-            end = self.tree.end()
+            end = self.tree.end
 
         current_degree = self.degree(node, begin=begin, end=begin)
         sd = SortedDict()
@@ -478,19 +516,19 @@ class IntervalDiGraph(IntervalGraph):
 
         # for each edge determine if the begin and/or end value is in specified time period
         for edge in self.edges(u=node, begin=begin, end=end):
-            if edge.begin >= begin:
+            if edge[2] >= begin:
                 # if begin is in specified time period, add to SortedDict, with +1 to indicate begin
-                sd.setdefault((edge.begin, 1), []).append(edge.data)
-            if edge.end < end:
+                sd.setdefault((edge[2], 1), []).append((edge[0], edge[1]))
+            if edge[3] < end:
                 # if begin is in specified time period, add to SortedDict, with -1 to indicate begin
-                sd.setdefault((edge.end, -1), []).append(edge.data)
+                sd.setdefault((edge[3], -1), []).append((edge[0], edge[1]))
         for edge in self.edges(v=node, begin=begin, end=end):
-            if edge.begin >= begin:
+            if edge[2] >= begin:
                 # if begin is in specified time period, add to SortedDict, with +1 to indicate begin
-                sd.setdefault((edge.begin, 1), []).append(edge.data)
-            if edge.end < end:
+                sd.setdefault((edge[2], 1), []).append((edge[0], edge[1]))
+            if edge[3] < end:
                 # if begin is in specified time period, add to SortedDict, with -1 to indicate begin
-                sd.setdefault((edge.end, -1), []).append(edge.data)
+                sd.setdefault((edge[3], -1), []).append((edge[0], edge[1]))
 
         for time in sd:
             for edge in sd[time]:
@@ -553,9 +591,9 @@ class IntervalDiGraph(IntervalGraph):
 
         # delta == True, return list of changes
         if begin == None:
-            begin = self.tree.begin()
+            begin = self.tree.begin
         if end == None:
-            end = self.tree.end()
+            end = self.tree.end
 
         current_degree = self.in_degree(node, begin=begin, end=begin)
         sd = SortedDict()
@@ -563,12 +601,12 @@ class IntervalDiGraph(IntervalGraph):
 
         # for each edge determine if the begin and/or end value is in specified time period
         for edge in self.edges(v=node, begin=begin, end=end):
-            if edge.begin >= begin:
+            if edge[2] >= begin:
                 # if begin is in specified time period, add to SortedDict, with +1 to indicate begin
-                sd.setdefault((edge.begin, 1), []).append(edge.data)
-            if edge.end < end:
+                sd.setdefault((edge[2], 1), []).append((edge[0], edge[1]))
+            if edge[3] < end:
                 # if begin is in specified time period, add to SortedDict, with -1 to indicate begin
-                sd.setdefault((edge.end, -1), []).append(edge.data)
+                sd.setdefault((edge[3], -1), []).append((edge[0], edge[1]))
 
         for time in sd:
             for edge in sd[time]:
@@ -631,9 +669,9 @@ class IntervalDiGraph(IntervalGraph):
 
         # delta == True, return list of changes
         if begin == None:
-            begin = self.tree.begin()
+            begin = self.tree.begin
         if end == None:
-            end = self.tree.end()
+            end = self.tree.end
 
         current_degree = self.out_degree(node, begin=begin, end=begin)
         sd = SortedDict()
@@ -641,12 +679,12 @@ class IntervalDiGraph(IntervalGraph):
 
         # for each edge determine if the begin and/or end value is in specified time period
         for edge in self.edges(u=node, begin=begin, end=end):
-            if edge.begin >= begin:
+            if edge[2] >= begin:
                 # if begin is in specified time period, add to SortedDict, with +1 to indicate begin
-                sd.setdefault((edge.begin, 1), []).append(edge.data)
-            if edge.end < end:
+                sd.setdefault((edge[2], 1), []).append((edge[0], edge[1]))
+            if edge[3] < end:
                 # if begin is in specified time period, add to SortedDict, with -1 to indicate begin
-                sd.setdefault((edge.end, -1), []).append(edge.data)
+                sd.setdefault((edge[3], -1), []).append((edge[0], edge[1]))
 
         for time in sd:
             for edge in sd[time]:
@@ -658,6 +696,98 @@ class IntervalDiGraph(IntervalGraph):
 
         return sorted(output)
 
+    def to_networkx_graph(self, begin, end, multigraph=False, edge_data=False, edge_interval_data=False, node_data=False):
+        """Return a networkx DiGraph or MultiDiGraph which includes all the nodes and
+        edges which have overlapping intervals with the given interval.
+
+        Wrapper function for IntervalDiGraph.to_subgraph. Refer to IntervalDiGraph.to_subgraph for full description.
+        """
+        return self.to_subgraph(begin=begin, end=end, multigraph=multigraph, edge_data=edge_data, edge_interval_data=edge_interval_data, node_data=node_data)
+
+    def to_subgraph(self, begin, end, multigraph=False, edge_data=False, edge_interval_data=False, node_data=False):
+        """Return a networkx DiGraph or MultiDiGraph which includes all the nodes and
+        edges which have overlapping intervals with the given interval.
+
+        Parameters
+        ----------
+        begin: int or float
+            Inclusive beginning time of the edge appearing in the interval graph.
+        end: int or float
+            Non-inclusive ending time of the edge appearing in the interval graph.
+            Must be bigger than or equal to begin.
+        multigraph: bool, optional (default= False)
+            If True, a networkx MultiGraph will be returned. If False, networkx Graph.
+        edge_data: bool, optional (default= False)
+            If True, edges will keep their attributes.
+        edge_interval_data: bool, optional (default= False)
+            If True, each edge's attribute will also include its begin and end interval data.
+            If `edge_data= True` and there already exist edge attributes with names begin and end,
+            they will be overwritten.
+        node_data : bool, optional (default= False)
+            If True, each node's attributes will be included.
+
+        See Also
+        --------
+        to_snapshots : divide the interval graph to snapshots
+
+        Notes
+        -----
+        If multigraph= False, and edge_data=True or edge_interval_data=True,
+        in case there are multiple edges, only one will show with one of the edge's attributes.
+
+        Note: nodes with no edges will not appear in any subgraph.
+
+        Examples
+        --------
+        >>> G = dnx.IntervalGraph()
+        >>> G.add_edges_from([(1, 2, 3, 10), (2, 4, 1, 11), (6, 4, 12, 19), (2, 4, 8, 15)])
+        >>> H = G.to_subgraph(4, 12)
+        >>> type(H)
+        <class 'networkx.classes.graph.DiGraph'>
+        >>> list(H.edges(data=True))
+        [(1, 2, {}), (2, 4, {})]
+
+        >>> H = G.to_subgraph(4, 12, edge_interval_data=True)
+        >>> type(H)
+        <class 'networkx.classes.graph.DiGraph'>
+        >>> list(H.edges(data=True))
+        [(1, 2, {'end': 10, 'begin': 3}), (2, 4, {'end': 15, 'begin': 8})]
+
+        >>> M = G.to_subgraph(4, 12, multigraph=True, edge_interval_data=True)
+        >>> type(M)
+        <class 'networkx.classes.multigraph.MultiDiGraph'>
+        >>> list(M.edges(data=True))
+        [(1, 2, {'end': 10, 'begin': 3}), (2, 4, {'end': 11, 'begin': 1}), (2, 4, {'end': 15, 'begin': 8})]
+        """
+        if begin and end and begin > end:
+            raise NetworkXError("IntervalGraph: interval end must be bigger than or equal to begin: "
+                                "begin: {}, end: {}.".format(begin, end))
+        iedges = self.tree[begin:end]
+
+        if multigraph:
+            G = MultiDiGraph()
+        else:
+            G = DiGraph()
+
+        if edge_data and edge_interval_data:
+            G.add_edges_from((iedge.data[0], iedge.data[1],
+                              dict(self._adj[iedge.data[0]][iedge], begin=iedge.begin, end=iedge.end))
+                             for iedge in iedges)
+        elif edge_data:
+            G.add_edges_from((iedge.data[0], iedge.data[1], self._adj[iedge.data[0]][iedge].copy())
+                             for iedge in iedges)
+        elif edge_interval_data:
+            G.add_edges_from((iedge.data[0], iedge.data[1], {'begin': iedge.begin, 'end': iedge.end})
+                             for iedge in iedges)
+        else:
+            G.add_edges_from((iedge.data[0], iedge.data[1]) for iedge in iedges)
+
+        # include node attributes
+        if node_data:
+            G.add_nodes_from((n, self._node[n].copy()) for n in G.nodes)
+
+        return G
+
     def __remove_iedge(self, iedge):
         """Remove the interval edge from the interval graph.
 
@@ -672,86 +802,12 @@ class IntervalDiGraph(IntervalGraph):
         --------
         >>> G = dnx.IntervalDiGraph()
         >>> G.add_edge(1, 2, 3, 10)
-        >>> iedge = Interval(3, 10, (1, 2))   # Interval(begin, end, (u, v))
+        >>> iedge = (1, 2, 3, 10)
         >>> G.__remove_iedge(iedge)
         """
-        self.tree.discard(iedge)
-        self._pred[iedge.data[0]].pop(iedge, None)
-        self._succ[iedge.data[1]].pop(iedge, None)
-
-    def __get_iedge_in_tree(self, u, v, begin, end):
-        """Return interval edge if found in the interval graph with the exact interval,
-        otherwise return None.
-
-        Parameters
-        ----------
-        u, v : nodes
-            Nodes can be, for example, strings or numbers.
-            Nodes must be hashable (and not None) Python objects.
-        begin : int or float
-            Inclusive beginning time of the edge appearing in the interval graph.
-        end : int or float
-            Non-inclusive ending time of the edge appearing in the interval graph.
-            Must be bigger than begin.
-
-        Examples
-        --------
-        >>> G = dnx.IntervalDiGraph()
-        >>> G.add_edge(1, 2, 3, 10)
-        >>> G.__get_iedge_in_tree(2, 1, 3, 10)
-        Interval(3, 10, (1, 2))
-        >>> G.__get_iedge_in_tree(2, 1, 4, 10)
-        None
-        """
-
-        temp_iedge = Interval(begin, end, (u, v))
-        if temp_iedge in self.tree:
-            return temp_iedge
-
-        return None
-
-    def __validate_interval(self, begin=None, end=None):
-        """Returns validated begin and end.
-        Replaces begin with the begin time of the graph if None is passed.
-        Replaces end with the end time of the graph + 1 (to make it inclusive) if None is passed.
-        Raises an exception if begin is larger than end.
-
-        Parameters
-        ----------
-        begin : int or float, optional
-            Inclusive beginning time of the edge appearing in the interval graph.
-        end : int or float, optional
-            Non-inclusive ending time of the edge appearing in the interval graph.
-        """
-        if begin is None:
-            begin = self.tree.begin()
-
-        if end is None:
-            end = self.tree.end() + 1
-
-        if begin > end:
-            raise NetworkXError("IntervalDiGraph: interval end must be bigger than or equal to begin: "
-                                "begin: {}, end: {}.".format(begin, end))
-
-        return begin, end
-
-    def __search_tree(self, begin=None, end=None):
-        """if begin and end are equal performs a point search on the tree,
-        otherwise an interval search is performed.
-
-       Parameters
-       ----------
-       begin: int or float, optional  (default= beginning of the entire interval graph)
-            Inclusive beginning time of the node appearing in the interval graph.
-        end: int or float, optional  (default= end of the entire interval graph + 1)
-            Non-inclusive ending time of the node appearing in the interval graph.
-            Must be bigger than or equal begin.
-            Note that the default value is shifted up by 1 to make it an inclusive end.
-       """
-        begin, end = self.__validate_interval(begin, end)
-        if begin != end:
-            return self.tree[begin:end]
-        return self.tree[begin]
+        self.tree.remove(iedge)
+        self._pred[iedge[0]][iedge[1]].pop(iedge, None)
+        self._succ[iedge[1]][iedge[0]].pop(iedge, None)
 
     @staticmethod
     def __overlaps_or_contains(iv, begin, end):
@@ -767,4 +823,10 @@ class IntervalDiGraph(IntervalGraph):
             Must be bigger than or equal begin.
        """
         # need to check for iv.contains(begin) in case begin == end
-        return iv.overlaps(begin, end) or iv.contains_point(begin)
+        if not begin and not end:
+            return True
+        if not begin:
+            return iv[2] < end
+        if not end:
+            return iv[3] > begin
+        return (iv[2] < end and iv[3] > begin) or iv[2] == begin
