@@ -1,6 +1,5 @@
 import dynetworkx as dnx
 import networkx as nx
-from intervaltree import Interval
 from networkx import from_numpy_matrix, from_numpy_array
 import os
 import numpy as np
@@ -153,8 +152,7 @@ def test_intervalgraph_add_edge():
     G.add_edge(1, 2, 3, 4)
     G.add_edge(1, 3, 4, 5, weight=7, capacity=15, length=342.7)
 
-    assert list(G.edges(data=True)) == [(Interval(4, 5, (1, 3)), {'capacity': 15, 'length': 342.7, 'weight': 7}),
-                                        (Interval(3, 4, (1, 2)), {})]
+    assert list(G.edges(data=True)) == [((1, 2, 3, 4), {}), ((1, 3, 4, 5), {'capacity': 15, 'length': 342.7, 'weight': 7})]
 
 
 def test_intervalgraph_add_edges_from():
@@ -162,10 +160,8 @@ def test_intervalgraph_add_edges_from():
     G.add_edges_from([(1, 2, 10, 11), (2, 4, 11, 12)])
     G.add_edges_from([(3, 4, 19, 20), (1, 4, 3, 4)], label='WN2898')
 
-    assert list(G.edges(data=True)) == [(Interval(19, 20, (3, 4)), {'label': 'WN2898'}),
-                                        (Interval(11, 12, (2, 4)), {}),
-                                        (Interval(10, 11, (1, 2)), {}),
-                                        (Interval(3, 4, (1, 4)), {'label': 'WN2898'})]
+    assert list(G.edges(data=True)) == [((1, 4, 3, 4), {'label': 'WN2898'}), ((1, 2, 10, 11), {}),
+                                        ((2, 4, 11, 12), {}), ((3, 4, 19, 20), {'label': 'WN2898'})]
 
 
 def test_intervalgraph_has_edge():
@@ -181,22 +177,21 @@ def test_intervalgraph_edges_default():
     G = dnx.IntervalGraph()
     G.add_edge(3, 4, 5, 6)
 
-    assert list(G.edges()) == [Interval(5, 6, (3, 4))]
+    assert list(G.edges()) == [(3, 4, 5, 6)]
 
 
 def test_intervalgraph_edges_slice():
     G = dnx.IntervalGraph()
     G.add_edges_from([(1, 2, 10, 11), (2, 4, 11, 12), (6, 4, 19, 20), (2, 4, 15, 16)])
 
-    assert list(G.edges(begin=10)) == [Interval(10, 11, (1, 2)), Interval(19, 20, (6, 4)),
-                                       Interval(11, 12, (2, 4)), Interval(15, 16, (2, 4))]
-    assert list(G.edges(end=11)) == [Interval(10, 11, (1, 2))]
-    assert list(G.edges(begin=11, end=15)) == [Interval(11, 12, (2, 4))]
-    assert list(G.edges(u=2)) == [Interval(10, 11, (1, 2)), Interval(11, 12, (2, 4)), Interval(15, 16, (2, 4))]
-    assert list(G.edges(v=2)) == [Interval(10, 11, (1, 2)), Interval(11, 12, (2, 4)), Interval(15, 16, (2, 4))]
-    assert list(G.edges(u=2, begin=11)) == [Interval(11, 12, (2, 4)), Interval(15, 16, (2, 4))]
-    assert list(G.edges(u=2, v=4, end=12)) == [Interval(11, 12, (2, 4))]
-    assert list(G.edges(u=1, v=2)) == [Interval(10, 11, (1, 2))]
+    assert list(G.edges(begin=10)) == [(1, 2, 10, 11), (2, 4, 11, 12), (2, 4, 15, 16), (6, 4, 19, 20)]
+    assert list(G.edges(end=11)) == [(1, 2, 10, 11)]
+    assert list(G.edges(begin=11, end=15)) == [(2, 4, 11, 12)]
+    assert list(G.edges(u=2)) == [(1, 2, 10, 11), (2, 4, 11, 12), (2, 4, 15, 16)]
+    assert list(G.edges(v=2)) == [(1, 2, 10, 11), (2, 4, 11, 12), (2, 4, 15, 16)]
+    assert list(G.edges(u=2, begin=11)) == [(2, 4, 11, 12), (2, 4, 15, 16)]
+    assert list(G.edges(u=2, v=4, end=12)) == [(2, 4, 11, 12)]
+    assert list(G.edges(u=1, v=2)) == [(1, 2, 10, 11)]
 
 
 def test_intervalgraph_edges_data():
@@ -205,16 +200,11 @@ def test_intervalgraph_edges_data():
     G.add_edge(1, 2, 10, 11, weight=10)
     G.add_edge(2, 6, 10, 11)
 
-    assert list(G.edges(data="weight")) == [(Interval(4, 5, (1, 3)), 8),
-                                            (Interval(10, 11, (2, 6)), None),
-                                            (Interval(10, 11, (1, 2)), 10)]
-    assert list(G.edges(data="weight", default=5)) == [(Interval(4, 5, (1, 3)), 8),
-                                                       (Interval(10, 11, (2, 6)), 5),
-                                                       (Interval(10, 11, (1, 2)), 10)]
-    assert list(G.edges(data=True)) == [(Interval(4, 5, (1, 3)), {'weight': 8, 'height': 18}),
-                                        (Interval(10, 11, (2, 6)), {}),
-                                        (Interval(10, 11, (1, 2)), {'weight': 10})]
-    assert list(G.edges(u=1, begin=2, end=9, data="weight")) == [(Interval(4, 5, (1, 3)), 8)]
+    assert list(G.edges(data="weight")) == [((1, 3, 4, 5), 8), ((1, 2, 10, 11), 10), ((2, 6, 10, 11), None)]
+    assert list(G.edges(data="weight", default=5)) == [((1, 3, 4, 5), 8), ((1, 2, 10, 11), 10), ((2, 6, 10, 11), 5)]
+    assert list(G.edges(data=True)) == [((1, 3, 4, 5), {'weight': 8, 'height': 18}),
+                                        ((1, 2, 10, 11), {'weight': 10}), ((2, 6, 10, 11), {})]
+    assert list(G.edges(u=1, begin=2, end=9, data="weight")) == [((1, 3, 4, 5), 8)]
 
 
 def test_intervalgraph_remove_edge_default():
@@ -264,26 +254,58 @@ def test_intervalgraph_from_networkx_graph_timestamp():
     assert actual.edges(data=True) == desired.edges(data=True)
 
 
+def test_intervalgraph_to_snapshot_graph():
+
+    # With interval edges
+    G = dnx.IntervalGraph()
+    G.add_edges_from([(1, 2, 10, 11), (2, 4, 11, 15), (6, 4, 19, 20), (2, 4, 13, 16), (3, 2, 10, 11)])
+    S = G.to_snapshot_graph(edge_data=False, edge_interval_data=True)
+    actual = []
+    for g in S:
+        actual.append(g.edges(data=True))
+    assert list(actual[0]) == [(1, 2, {'begin': 10, 'end': 11}), (2, 3, {'begin': 10, 'end': 11})]
+    assert list(actual[1]) == [(2, 4, {'begin': 11, 'end': 15})]
+    assert list(actual[2]) == [(2, 4, {'begin': 13, 'end': 16})]
+    assert list(actual[3]) == [(2, 4, {'begin': 13, 'end': 16})]
+    assert list(actual[4]) == []
+    assert list(actual[5]) == [(6, 4, {'begin': 19, 'end': 20})]
+
+    # With impulse edges
+    G2 = dnx.IntervalGraph()
+    G2.add_edges_from([(1, 2, 11, 11), (2, 4, 15, 15), (6, 4, 19, 19)])
+    S2 = G2.to_snapshot_graph(edge_data=False, edge_interval_data=True)
+    actual2 = []
+    for g in S2:
+        actual2.append(g.edges(data=True))
+    assert list(actual2[0]) == [(1, 2, {'begin': 11, 'end': 11})]
+    assert list(actual2[1]) == [(2, 4, {'begin': 15, 'end': 15})]
+    assert list(actual2[2]) == [(6, 4, {'begin': 19, 'end': 19})]
+
+
 def test_intervalgraph_from_snapshots_default():
     desired = dnx.IntervalGraph()
     desired.add_edge(1, 2, 0, 1)
+    desired.add_edge(1, 2, 2, 3)
     desired.add_edge(6, 7, 0, 1)
     desired.add_edge(5, 6, 0, 1)
     desired.add_edge(1, 4, 1, 2)
     desired.add_edge(1, 3, 0, 2)
     desired.add_edge(10, 11, 0, 2)
-    desired.add_edge(8, 9, 0, 2, weight=1)
+    desired.add_edge(8, 9, 0, 3, weight=1)
 
     sg = dnx.SnapshotGraph()
-    sg.add_snapshot([(1, 2), (1, 3)])
-    sg.add_snapshot([(1, 4), (1, 3)])
+    sg.add_snapshot([(1, 2), (1, 3)], time=0)
+    sg.add_snapshot([(1, 4), (1, 3)], time=1)
+    sg.add_snapshot([(1, 2)], time=2)
     sg.add_edges_from([(5, 6), (7, 6)], [0])
     sg.add_edges_from([(8, 9), (10, 11)], [0, 1])
     sg.add_edges_from([(8, 9)], weight=1)
 
-    actual = dnx.IntervalGraph.from_snapshots(sg)
+    actual = dnx.IntervalGraph.from_snapshot_graph(sg)
 
-    assert sorted(actual.edges(data=True)) == sorted(desired.edges(data=True))
+    assert len(actual.edges()) == len(desired.edges())
+    for edge in actual.edges(data=True):
+        assert edge in desired.edges(data=True)
 
 
 def test_intervalgraph_from_snapshots_begin():
@@ -297,15 +319,17 @@ def test_intervalgraph_from_snapshots_begin():
     desired.add_edge(8, 9, 10, 12, weight=1)
 
     sg = dnx.SnapshotGraph()
-    sg.add_snapshot([(1, 2), (1, 3)])
-    sg.add_snapshot([(1, 4), (1, 3)])
+    sg.add_snapshot([(1, 2), (1, 3)], time=0)
+    sg.add_snapshot([(1, 4), (1, 3)], time=1)
     sg.add_edges_from([(5, 6), (7, 6)], [0])
     sg.add_edges_from([(8, 9), (10, 11)], [0, 1])
     sg.add_edges_from([(8, 9)], weight=1)
 
-    actual = dnx.IntervalGraph.from_snapshots(sg, begin=10)
+    actual = dnx.IntervalGraph.from_snapshot_graph(sg, begin=10)
 
-    assert sorted(actual.edges(data=True)) == sorted(desired.edges(data=True))
+    assert len(actual.edges()) == len(desired.edges())
+    for edge in actual.edges(data=True):
+        assert edge in desired.edges(data=True)
 
 
 def test_intervalgraph_from_snapshots_period():
@@ -319,24 +343,46 @@ def test_intervalgraph_from_snapshots_period():
     desired.add_edge(8, 9, 0, 4, weight=1)
 
     sg = dnx.SnapshotGraph()
-    sg.add_snapshot([(1, 2), (1, 3)])
-    sg.add_snapshot([(1, 4), (1, 3)])
+    sg.add_snapshot([(1, 2), (1, 3)], time=0)
+    sg.add_snapshot([(1, 4), (1, 3)], time=1)
     sg.add_edges_from([(5, 6), (7, 6)], [0])
     sg.add_edges_from([(8, 9), (10, 11)], [0, 1])
     sg.add_edges_from([(8, 9)], weight=1)
 
-    actual = dnx.IntervalGraph.from_snapshots(sg, period=2)
+    actual = dnx.IntervalGraph.from_snapshot_graph(sg, period=2)
 
-    assert sorted(actual.edges(data=True)) == sorted(desired.edges(data=True))
+    assert len(actual.edges()) == len(desired.edges())
+    for edge in actual.edges(data=True):
+        assert edge in desired.edges(data=True)
 
 
 def test_intervalgraph_to_snapshots_default():
     G = dnx.IntervalGraph()
     G.add_edges_from([(1, 2, 3, 6), (2, 4, 5, 11), (6, 4, 19, 20), (2, 4, 15, 16)])
-    S = G.to_snapshots(2)
+    S = G.to_snapshots(2, return_length=True)
 
-    assert sorted(list(S[0].edges())) == [(1, 2), (2, 4)]
-    assert sorted(list(S[1].edges())) == [(4, 2), (6, 4)]
+    assert len(S[0]) == 2
+    assert S[1] == 8.5
+    assert sorted(list(S[0][0].edges())) == [(1, 2), (2, 4)]
+    assert sorted(list(S[0][1].edges())) == [(2, 4), (4, 6)]
+
+
+def test_interval_graph_to_snapshots_len():
+    G = dnx.IntervalGraph()
+    G.add_edges_from([(1, 2, 3, 6), (2, 4, 5, 11), (6, 4, 19, 20), (2, 4, 15, 16)])
+    S = G.to_snapshots(length_of_snapshots=2, return_length=True)
+
+    assert len(S[0]) == 9
+    assert S[1] == 2
+    assert sorted(list(S[0][0].edges())) == [(1, 2)]
+    assert sorted(list(S[0][1].edges())) == [(1, 2), (2, 4)]
+    assert sorted(list(S[0][2].edges())) == [(2, 4)]
+    assert sorted(list(S[0][3].edges())) == [(2, 4)]
+    assert sorted(list(S[0][4].edges())) == []
+    assert sorted(list(S[0][5].edges())) == []
+    assert sorted(list(S[0][6].edges())) == [(2, 4)]
+    assert sorted(list(S[0][7].edges())) == []
+    assert sorted(list(S[0][8].edges())) == [(6, 4)]
 
 
 def test_intervalgraph_to_subgraph_default():
@@ -377,6 +423,16 @@ def test_intervalgraph_to_subgraph_edge_interval_data():
     assert isinstance(H, nx.classes.graph.Graph)
 
 
+def test_intervalgraph_to_subgraph_edge_data_and_edge_interval_data():
+    G = dnx.IntervalGraph()
+    G.add_edges_from([(1, 2, 3, 6), (2, 4, 5, 11), (6, 4, 19, 20), (2, 4, 15, 16)], weight=1.5)
+    H = G.to_subgraph(4, 12, edge_data=True, edge_interval_data=True)
+
+    assert list(H.edges(data=True)) == [(1, 2, {'weight': 1.5, 'begin': 3, 'end': 6}),
+                                        (2, 4, {'weight': 1.5, 'begin': 5, 'end': 11})]
+    assert isinstance(H, nx.classes.graph.Graph)
+
+
 def test_intervalgraph_to_subgraph_multigraph():
     G = dnx.IntervalGraph()
     G.add_edges_from([(1, 2, 3, 6), (2, 4, 5, 11), (6, 4, 19, 20), (2, 4, 15, 16)])
@@ -415,12 +471,12 @@ def test_intervalgraph_load_from_text_delimiter():
 def test_intervalgraph_load_from_text_inputtypes():
     path = os.path.join(current_dir, 'inputoutput_text/intervalgraph_load_from_text_default.txt')
     desired = dnx.IntervalGraph()
-    desired.add_edge('1', '2', '3.0', '4.0')
-    desired.add_edge('5', '6', '7.0', '8.0')
-    desired.add_edge('9', '10', '11.0', '12.0', weight=1.0)
-    desired.add_edge('13', '14', '15.0', '16.0', weight=2.0)
+    desired.add_edge('1', '2', 3.0, 4.0)
+    desired.add_edge('5', '6', 7.0, 8.0)
+    desired.add_edge('9', '10', 11.0, 12.0, weight=1.0)
+    desired.add_edge('13', '14', 15.0, 16.0, weight=2.0)
 
-    actual = dnx.IntervalGraph.load_from_txt(path, nodetype=str, intervaltype=str)
+    actual = dnx.IntervalGraph.load_from_txt(path, nodetype=str, intervaltype=float)
 
     assert actual.edges(data=True) == desired.edges(data=True)
 
@@ -475,4 +531,4 @@ def test_intervalgraph_save_to_text_delimiter():
     with open(output_path, 'r') as output_file:
         actual = output_file.read()
 
-    assert actual == desired
\ No newline at end of file
+    assert actual == desired
